@@ -7,6 +7,9 @@ from beanie.odm.actions import (
     Replace,
     SaveChanges,
     ValidateOnSave,
+    Before,
+    After,
+    Delete,
 )
 from beanie.odm.bulk import BulkWriter
 from beanie.odm.fields import (
@@ -16,16 +19,19 @@ from beanie.odm.fields import (
     WriteRules,
     DeleteRules,
 )
+from beanie.odm.settings.timeseries import TimeSeriesConfig, Granularity
 from beanie.odm.utils.general import init_beanie
 from beanie.odm.documents import Document
 
-__version__ = "1.8.10"
+__version__ = "1.10.7"
 __all__ = [
     # ODM
     "Document",
     "init_beanie",
     "PydanticObjectId",
     "Indexed",
+    "TimeSeriesConfig",
+    "Granularity",
     # Actions
     "before_event",
     "after_event",
@@ -33,6 +39,9 @@ __all__ = [
     "Replace",
     "SaveChanges",
     "ValidateOnSave",
+    "Delete",
+    "Before",
+    "After",
     # Bulk Write
     "BulkWriter",
     # Migrations
