@@ -1,5 +1,6 @@
 import asyncio
 import inspect
+from typing import ClassVar, AbstractSet
 from typing import (
     Dict,
     Optional,
@@ -25,7 +26,6 @@ from pydantic import (
     Field,
 )
 from pydantic.main import BaseModel
-from pydantic.types import ClassVar
 from pymongo import InsertOne
 from pymongo.client_session import ClientSession
 from pymongo.results import (
@@ -41,7 +41,12 @@ from beanie.exceptions import (
     DocumentWasNotSaved,
     NotSupported,
 )
-from beanie.odm.actions import EventTypes, wrap_with_actions, ActionRegistry
+from beanie.odm.actions import (
+    EventTypes,
+    wrap_with_actions,
+    ActionRegistry,
+    ActionDirections,
+)
 from beanie.odm.bulk import BulkWriter, Operation
 from beanie.odm.cache import LRUCache
 from beanie.odm.enums import SortDirection
@@ -156,6 +161,7 @@ class Document(BaseModel, UpdateMethods):
         *,
         link_rule: WriteRules = WriteRules.DO_NOTHING,
         session: Optional[ClientSession] = None,
+        skip_actions: Optional[List[Union[ActionDirections, str]]] = None,
     ) -> DocType:
         """
         Insert the document (self) to the collection
@@ -267,6 +273,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> Optional[DocType]:
         """
         Get document by id, returns None if document does not exist
@@ -274,6 +281,7 @@ class Document(BaseModel, UpdateMethods):
         :param document_id: PydanticObjectId - document id
         :param session: Optional[ClientSession] - pymongo session
         :param ignore_cache: bool - ignore cache (if it is turned on)
+        :param **pymongo_kwargs: pymongo native parameters for find operation
         :return: Union["Document", None]
         """
         if not isinstance(document_id, cls.__fields__["id"].type_):
@@ -283,6 +291,7 @@ class Document(BaseModel, UpdateMethods):
             session=session,
             ignore_cache=ignore_cache,
             fetch_links=fetch_links,
+            **pymongo_kwargs,
         )
 
     @overload
@@ -294,6 +303,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> FindOne[DocType]:
         ...
 
@@ -306,6 +316,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> FindOne[DocumentProjectionType]:
         ...
 
@@ -317,6 +328,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> Union[FindOne[DocType], FindOne[DocumentProjectionType]]:
         """
         Find one document by criteria.
@@ -327,6 +339,7 @@ class Document(BaseModel, UpdateMethods):
         :param projection_model: Optional[Type[BaseModel]] - projection model
         :param session: Optional[ClientSession] - pymongo session instance
         :param ignore_cache: bool
+        :param **pymongo_kwargs: pymongo native parameters for find operation (if Document class contains links, this parameter must fit the respective parameter of the aggregate MongoDB function)
         :return: [FindOne](https://roman-right.github.io/beanie/api/queries/#findone) - find query instance
         """
         return cls._find_one_query_class(document_model=cls).find_one(
@@ -335,6 +348,7 @@ class Document(BaseModel, UpdateMethods):
             session=session,
             ignore_cache=ignore_cache,
             fetch_links=fetch_links,
+            **pymongo_kwargs,
         )
 
     @overload
@@ -349,6 +363,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocType]:
         ...
 
@@ -364,6 +379,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocumentProjectionType]:
         ...
 
@@ -378,6 +394,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> Union[FindMany[DocType], FindMany[DocumentProjectionType]]:
         """
         Find many documents by criteria.
@@ -390,6 +407,7 @@ class Document(BaseModel, UpdateMethods):
         :param projection_model: Optional[Type[BaseModel]] - projection model
         :param session: Optional[ClientSession] - pymongo session
         :param ignore_cache: bool
+        :param **pymongo_kwargs: pymongo native parameters for find operation (if Document class contains links, this parameter must fit the respective parameter of the aggregate MongoDB function)
         :return: [FindMany](https://roman-right.github.io/beanie/api/queries/#findmany) - query instance
         """
         return cls._find_many_query_class(document_model=cls).find_many(
@@ -401,6 +419,7 @@ class Document(BaseModel, UpdateMethods):
             session=session,
             ignore_cache=ignore_cache,
             fetch_links=fetch_links,
+            **pymongo_kwargs,
         )
 
     @overload
@@ -415,6 +434,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocType]:
         ...
 
@@ -430,6 +450,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocumentProjectionType]:
         ...
 
@@ -444,6 +465,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> Union[FindMany[DocType], FindMany[DocumentProjectionType]]:
         """
         The same as find_many
@@ -457,6 +479,7 @@ class Document(BaseModel, UpdateMethods):
             session=session,
             ignore_cache=ignore_cache,
             fetch_links=fetch_links,
+            **pymongo_kwargs,
         )
 
     @overload
@@ -469,6 +492,7 @@ class Document(BaseModel, UpdateMethods):
         projection_model: None = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocType]:
         ...
 
@@ -482,6 +506,7 @@ class Document(BaseModel, UpdateMethods):
         projection_model: Optional[Type[DocumentProjectionType]] = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocumentProjectionType]:
         ...
 
@@ -494,6 +519,7 @@ class Document(BaseModel, UpdateMethods):
         projection_model: Optional[Type[DocumentProjectionType]] = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> Union[FindMany[DocType], FindMany[DocumentProjectionType]]:
         """
         Get all the documents
@@ -503,6 +529,7 @@ class Document(BaseModel, UpdateMethods):
         :param sort: Union[None, str, List[Tuple[str, SortDirection]]] - A key or a list of (key, direction) pairs specifying the sort order for this query.
         :param projection_model: Optional[Type[BaseModel]] - projection model
         :param session: Optional[ClientSession] - pymongo session
+        :param **pymongo_kwargs: pymongo native parameters for find operation (if Document class contains links, this parameter must fit the respective parameter of the aggregate MongoDB function)
         :return: [FindMany](https://roman-right.github.io/beanie/api/queries/#findmany) - query instance
         """
         return cls.find_many(
@@ -513,6 +540,7 @@ class Document(BaseModel, UpdateMethods):
             projection_model=projection_model,
             session=session,
             ignore_cache=ignore_cache,
+            **pymongo_kwargs,
         )
 
     @overload
@@ -525,6 +553,7 @@ class Document(BaseModel, UpdateMethods):
         sort: Union[None, str, List[Tuple[str, SortDirection]]] = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocType]:
         ...
 
@@ -538,6 +567,7 @@ class Document(BaseModel, UpdateMethods):
         sort: Union[None, str, List[Tuple[str, SortDirection]]] = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> FindMany[DocumentProjectionType]:
         ...
 
@@ -550,6 +580,7 @@ class Document(BaseModel, UpdateMethods):
         sort: Union[None, str, List[Tuple[str, SortDirection]]] = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> Union[FindMany[DocType], FindMany[DocumentProjectionType]]:
         """
         the same as find_all
@@ -561,6 +592,7 @@ class Document(BaseModel, UpdateMethods):
             projection_model=projection_model,
             session=session,
             ignore_cache=ignore_cache,
+            **pymongo_kwargs,
         )
 
     @wrap_with_actions(EventTypes.REPLACE)
@@ -573,6 +605,7 @@ class Document(BaseModel, UpdateMethods):
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
         link_rule: WriteRules = WriteRules.DO_NOTHING,
+        skip_actions: Optional[List[Union[ActionDirections, str]]] = None,
     ) -> DocType:
         """
         Fully update the document in the database
@@ -637,6 +670,7 @@ class Document(BaseModel, UpdateMethods):
         self: DocType,
         session: Optional[ClientSession] = None,
         link_rule: WriteRules = WriteRules.DO_NOTHING,
+        **kwargs,
     ) -> DocType:
         """
         Update an existing model in the database or insert it if it does not yet exist.
@@ -665,9 +699,9 @@ class Document(BaseModel, UpdateMethods):
                                 )
 
         try:
-            return await self.replace(session=session)
+            return await self.replace(session=session, **kwargs)
         except (ValueError, DocumentNotFound):
-            return await self.insert(session=session)
+            return await self.insert(session=session, **kwargs)
 
     @saved_state_needed
     @wrap_with_actions(EventTypes.SAVE_CHANGES)
@@ -677,6 +711,7 @@ class Document(BaseModel, UpdateMethods):
         ignore_revision: bool = False,
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        skip_actions: Optional[List[Union[ActionDirections, str]]] = None,
     ) -> None:
         """
         Save changes.
@@ -724,6 +759,7 @@ class Document(BaseModel, UpdateMethods):
         ignore_revision: bool = False,
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> None:
         """
         Partially update the document in the database
@@ -732,6 +768,7 @@ class Document(BaseModel, UpdateMethods):
         :param session: ClientSession - pymongo session.
         :param ignore_revision: bool - force update. Will update even if revision id is not the same, as stored
         :param bulk_writer: "BulkWriter" - Beanie bulk writer
+        :param **pymongo_kwargs: pymongo native parameters for update operation
         :return: None
         """
         use_revision_id = self.get_settings().model_settings.use_revision
@@ -742,13 +779,13 @@ class Document(BaseModel, UpdateMethods):
             find_query["revision_id"] = self._previous_revision_id
 
         result = await self.find_one(find_query).update(
-            *args, session=session, bulk_writer=bulk_writer
+            *args, session=session, bulk_writer=bulk_writer, **pymongo_kwargs
         )
 
         if (
             use_revision_id
             and not ignore_revision
-            and result.modified_count == 0
+            and result.matched_count == 0
         ):
             raise RevisionIdWasChanged
         await self._sync()
@@ -759,6 +796,7 @@ class Document(BaseModel, UpdateMethods):
         *args: Union[dict, Mapping],
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> UpdateMany:
         """
         Partially update all the documents
@@ -766,17 +804,21 @@ class Document(BaseModel, UpdateMethods):
         :param args: *Union[dict, Mapping] - the modifications to apply.
         :param session: ClientSession - pymongo session.
         :param bulk_writer: "BulkWriter" - Beanie bulk writer
+        :param **pymongo_kwargs: pymongo native parameters for find operation
         :return: UpdateMany query
         """
         return cls.find_all().update_many(
-            *args, session=session, bulk_writer=bulk_writer
+            *args, session=session, bulk_writer=bulk_writer, **pymongo_kwargs
         )
 
+    @wrap_with_actions(EventTypes.DELETE)
     async def delete(
         self,
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
         link_rule: DeleteRules = DeleteRules.DO_NOTHING,
+        skip_actions: Optional[List[Union[ActionDirections, str]]] = None,
+        **pymongo_kwargs,
     ) -> Optional[DeleteResult]:
         """
         Delete the document
@@ -784,6 +826,7 @@ class Document(BaseModel, UpdateMethods):
         :param session: Optional[ClientSession] - pymongo session.
         :param bulk_writer: "BulkWriter" - Beanie bulk writer
         :param link_rule: DeleteRules - rules for link fields
+        :param **pymongo_kwargs: pymongo native parameters for delete operation
         :return: Optional[DeleteResult] - pymongo DeleteResult instance.
         """
 
@@ -798,17 +841,19 @@ class Document(BaseModel, UpdateMethods):
                     ]:
                         if isinstance(value, Document):
                             await value.delete(
-                                link_rule=DeleteRules.DELETE_LINKS
+                                link_rule=DeleteRules.DELETE_LINKS,
+                                **pymongo_kwargs,
                             )
                     if field_info.link_type == LinkTypes.LIST:
                         for obj in value:
                             if isinstance(obj, Document):
                                 await obj.delete(
-                                    link_rule=DeleteRules.DELETE_LINKS
+                                    link_rule=DeleteRules.DELETE_LINKS,
+                                    **pymongo_kwargs,
                                 )
 
         return await self.find_one({"_id": self.id}).delete(
-            session=session, bulk_writer=bulk_writer
+            session=session, bulk_writer=bulk_writer, **pymongo_kwargs
         )
 
     @classmethod
@@ -816,16 +861,18 @@ class Document(BaseModel, UpdateMethods):
         cls,
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> Optional[DeleteResult]:
         """
         Delete all the documents
 
         :param session: Optional[ClientSession] - pymongo session.
         :param bulk_writer: "BulkWriter" - Beanie bulk writer
+        :param **pymongo_kwargs: pymongo native parameters for delete operation
         :return: Optional[DeleteResult] - pymongo DeleteResult instance.
         """
         return await cls.find_all().delete(
-            session=session, bulk_writer=bulk_writer
+            session=session, bulk_writer=bulk_writer, **pymongo_kwargs
         )
 
     @overload
@@ -836,6 +883,7 @@ class Document(BaseModel, UpdateMethods):
         projection_model: None = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> AggregationQuery[Dict[str, Any]]:
         ...
 
@@ -847,6 +895,7 @@ class Document(BaseModel, UpdateMethods):
         projection_model: Type[DocumentProjectionType],
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> AggregationQuery[DocumentProjectionType]:
         ...
 
@@ -857,6 +906,7 @@ class Document(BaseModel, UpdateMethods):
         projection_model: Optional[Type[DocumentProjectionType]] = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> Union[
         AggregationQuery[Dict[str, Any]],
         AggregationQuery[DocumentProjectionType],
@@ -868,6 +918,7 @@ class Document(BaseModel, UpdateMethods):
         :param projection_model: Type[BaseModel]
         :param session: Optional[ClientSession]
         :param ignore_cache: bool
+        :param **pymongo_kwargs: pymongo native parameters for aggregate operation
         :return: [AggregationQuery](https://roman-right.github.io/beanie/api/queries/#aggregationquery)
         """
         return cls.find_all().aggregate(
@@ -875,6 +926,7 @@ class Document(BaseModel, UpdateMethods):
             projection_model=projection_model,
             session=session,
             ignore_cache=ignore_cache,
+            **pymongo_kwargs,
         )
 
     @classmethod
@@ -897,6 +949,16 @@ class Document(BaseModel, UpdateMethods):
         """
         return cls.get_settings().model_settings.use_state_management
 
+    @classmethod
+    def state_management_replace_objects(cls) -> bool:
+        """
+        Should objects be replaced when using state management
+        :return: bool
+        """
+        return (
+            cls.get_settings().model_settings.state_management_replace_objects
+        )
+
     def _save_state(self) -> None:
         """
         Save current document state. Internal method
@@ -947,12 +1009,10 @@ class Document(BaseModel, UpdateMethods):
 
         for field_name, field_value in new_dict.items():
             if field_value != old_dict.get(field_name):
-                if not (
+                if not self.state_management_replace_objects() and (
                     isinstance(field_value, dict)
                     and isinstance(old_dict.get(field_name), dict)
                 ):
-                    updates[field_name] = field_value
-                else:
                     if old_dict.get(field_name) is None:
                         updates[field_name] = field_value
                     elif isinstance(field_value, dict) and isinstance(
@@ -966,6 +1026,8 @@ class Document(BaseModel, UpdateMethods):
 
                         for k, v in field_data.items():
                             updates[f"{field_name}.{k}"] = v
+                else:
+                    updates[field_name] = field_value
 
         return updates
 
@@ -1133,6 +1195,7 @@ class Document(BaseModel, UpdateMethods):
         exclude: Union["AbstractSetIntStr", "MappingIntStrAny"] = None,
         by_alias: bool = False,
         skip_defaults: bool = None,
+        exclude_hidden: bool = True,
         exclude_unset: bool = False,
         exclude_defaults: bool = False,
         exclude_none: bool = False,
@@ -1141,8 +1204,13 @@ class Document(BaseModel, UpdateMethods):
         Overriding of the respective method from Pydantic
         Hides fields, marked as "hidden
         """
-        if exclude is None:
-            exclude = self._hidden_fields
+        if exclude_hidden:
+            if isinstance(exclude, AbstractSet):
+                exclude = {*self._hidden_fields, *exclude}
+            elif isinstance(exclude, Mapping):
+                exclude = dict({k: True for k in self._hidden_fields}, **exclude)  # type: ignore
+            elif exclude is None:
+                exclude = self._hidden_fields
         return super().dict(
             include=include,
             exclude=exclude,
@@ -1154,7 +1222,7 @@ class Document(BaseModel, UpdateMethods):
         )
 
     @wrap_with_actions(event_type=EventTypes.VALIDATE_ON_SAVE)
-    async def validate_self(self):
+    async def validate_self(self, *args, **kwargs):
         # TODO it can be sync, but needs some actions controller improvements
         if self.get_settings().model_settings.validate_on_save:
             self.parse_obj(self)
