@@ -34,7 +34,6 @@ from beanie.odm.operators.find.logical import And
 from beanie.odm.queries.aggregation import AggregationQuery
 from beanie.odm.queries.cursor import BaseCursorQuery
 from beanie.odm.queries.delete import (
-    DeleteQuery,
     DeleteMany,
     DeleteOne,
 )
@@ -47,6 +46,7 @@ from beanie.odm.utils.encoder import Encoder
 from beanie.odm.utils.find import construct_lookup_queries
 from beanie.odm.utils.parsing import parse_obj
 from beanie.odm.utils.projection import get_projection
+from beanie.odm.utils.relations import convert_ids
 
 if TYPE_CHECKING:
     from beanie.odm.documents import DocType
@@ -68,9 +68,7 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
     UpdateQueryType: Union[
         Type[UpdateQuery], Type[UpdateMany], Type[UpdateOne]
     ] = UpdateQuery
-    DeleteQueryType: Union[
-        Type[DeleteOne], Type[DeleteMany], Type[DeleteQuery]
-    ] = DeleteQuery
+    DeleteQueryType: Union[Type[DeleteOne], Type[DeleteMany]] = DeleteMany
     AggregationQueryType = AggregationQuery
 
     def __init__(self, document_model: Type["DocType"]):
@@ -86,8 +84,14 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
             self.document_model.get_settings().model_settings.bson_encoders
         )
         self.fetch_links: bool = False
+        self.pymongo_kwargs: Dict[str, Any] = {}
 
     def get_filter_query(self) -> Mapping[str, Any]:
+        """
+
+        Returns: MongoDB filter query
+
+        """
         if self.find_expressions:
             return Encoder(custom_encoders=self.encoders).encode(
                 And(*self.find_expressions).query
@@ -100,7 +104,7 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
         *args: Mapping[str, Any],
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
-        **kwargs,
+        **pymongo_kwargs,
     ):
         """
         Create Update with modifications query
@@ -117,7 +121,7 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
                 document_model=self.document_model,
                 find_query=self.get_filter_query(),
             )
-            .update(*args, bulk_writer=bulk_writer)
+            .update(*args, bulk_writer=bulk_writer, **pymongo_kwargs)
             .set_session(session=self.session)
         )
 
@@ -126,6 +130,8 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
         *args: Mapping[str, Any],
         on_insert: "DocType",
         session: Optional[ClientSession] = None,
+        bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ):
         """
         Create Update with modifications query
@@ -143,7 +149,12 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
                 document_model=self.document_model,
                 find_query=self.get_filter_query(),
             )
-            .upsert(*args, on_insert=on_insert)
+            .upsert(
+                *args,
+                on_insert=on_insert,
+                bulk_writer=bulk_writer,
+                **pymongo_kwargs,
+            )
             .set_session(session=self.session)
         )
 
@@ -151,6 +162,7 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
         self,
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> Union[DeleteOne, DeleteMany]:
         """
         Provide search criteria to the Delete query
@@ -163,12 +175,10 @@ class FindQuery(Generic[FindQueryResultType], UpdateMethods, SessionMethods):
             document_model=self.document_model,
             find_query=self.get_filter_query(),
             bulk_writer=bulk_writer,
+            **pymongo_kwargs,
         ).set_session(session=session)
 
-    def project(
-        self,
-        projection_model,
-    ):
+    def project(self, projection_model):
         """
         Apply projection parameter
         :param projection_model: Optional[Type[BaseModel]] - projection model
@@ -237,6 +247,7 @@ class FindMany(
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> "FindMany[FindQueryResultType]":
         ...
 
@@ -251,6 +262,7 @@ class FindMany(
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> "FindMany[FindQueryProjectionType]":
         ...
 
@@ -264,6 +276,7 @@ class FindMany(
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> Union[
         "FindMany[FindQueryResultType]", "FindMany[FindQueryProjectionType]"
     ]:
@@ -279,6 +292,7 @@ class FindMany(
         :param projection_model: Optional[Type[BaseModel]] - projection model
         :param session: Optional[ClientSession] - pymongo session
         :param ignore_cache: bool
+        :param **pymongo_kwargs: pymongo native parameters for find operation (if Document class contains links, this parameter must fit the respective parameter of the aggregate MongoDB function)
         :return: FindMany - query instance
         """
         self.find_expressions += args  # type: ignore # bool workaround
@@ -289,6 +303,7 @@ class FindMany(
         self.set_session(session=session)
         self.ignore_cache = ignore_cache
         self.fetch_links = fetch_links
+        self.pymongo_kwargs.update(pymongo_kwargs)
         return self
 
     # TODO probably merge FindOne and FindMany to one class to avoid this
@@ -334,6 +349,7 @@ class FindMany(
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> "FindMany[FindQueryResultType]":
         ...
 
@@ -348,6 +364,7 @@ class FindMany(
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> "FindMany[FindQueryProjectionType]":
         ...
 
@@ -361,6 +378,7 @@ class FindMany(
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> Union[
         "FindMany[FindQueryResultType]", "FindMany[FindQueryProjectionType]"
     ]:
@@ -376,6 +394,7 @@ class FindMany(
             session=session,
             ignore_cache=ignore_cache,
             fetch_links=fetch_links,
+            **pymongo_kwargs,
         )
 
     def sort(
@@ -443,6 +462,7 @@ class FindMany(
         *args: Mapping[str, Any],
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> UpdateMany:
         """
         Provide search criteria to the
@@ -454,13 +474,19 @@ class FindMany(
         """
         return cast(
             UpdateMany,
-            self.update(*args, session=session, bulk_writer=bulk_writer),
+            self.update(
+                *args,
+                session=session,
+                bulk_writer=bulk_writer,
+                **pymongo_kwargs,
+            ),
         )
 
     def delete_many(
         self,
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> DeleteMany:
         """
         Provide search criteria to the [DeleteMany](https://roman-right.github.io/beanie/api/queries/#deletemany) query
@@ -472,7 +498,10 @@ class FindMany(
         # This is because delete may also return a DeleteOne type in general, and mypy can not be sure in this case
         # See https://mypy.readthedocs.io/en/stable/common_issues.html#narrowing-and-inner-functions
         return cast(
-            DeleteMany, self.delete(session=session, bulk_writer=bulk_writer)
+            DeleteMany,
+            self.delete(
+                session=session, bulk_writer=bulk_writer, **pymongo_kwargs
+            ),
         )
 
     @overload
@@ -482,6 +511,7 @@ class FindMany(
         projection_model: None = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> AggregationQuery[Dict[str, Any]]:
         ...
 
@@ -492,6 +522,7 @@ class FindMany(
         projection_model: Type[FindQueryProjectionType],
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> AggregationQuery[FindQueryProjectionType]:
         ...
 
@@ -501,6 +532,7 @@ class FindMany(
         projection_model: Optional[Type[FindQueryProjectionType]] = None,
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
+        **pymongo_kwargs,
     ) -> Union[
         AggregationQuery[Dict[str, Any]],
         AggregationQuery[FindQueryProjectionType],
@@ -522,6 +554,7 @@ class FindMany(
             projection_model=projection_model,
             find_query=self.get_filter_query(),
             ignore_cache=ignore_cache,
+            **pymongo_kwargs,
         ).set_session(session=self.session)
 
     @property
@@ -559,6 +592,14 @@ class FindMany(
 
     @property
     def motor_cursor(self):
+        if self.document_model.get_link_fields() is not None:
+            for i, query in enumerate(self.find_expressions):
+                self.find_expressions[i] = convert_ids(
+                    query,
+                    doc=self.document_model,
+                    fetch_links=self.fetch_links,
+                )
+
         if self.fetch_links:
             aggregation_pipeline: List[
                 Dict[str, Any]
@@ -579,7 +620,9 @@ class FindMany(
                 {"$project": get_projection(self.projection_model)}
             )
             return self.document_model.get_motor_collection().aggregate(
-                aggregation_pipeline, session=self.session
+                aggregation_pipeline,
+                session=self.session,
+                **self.pymongo_kwargs,
             )
 
         return self.document_model.get_motor_collection().find(
@@ -589,8 +632,18 @@ class FindMany(
             skip=self.skip_number,
             limit=self.limit_number,
             session=self.session,
+            **self.pymongo_kwargs,
         )
 
+    async def first_or_none(self) -> Optional[FindQueryResultType]:
+        """
+        Returns the first found element or None if no elements were found
+        """
+        res = await self.limit(1).to_list()
+        if not res:
+            return None
+        return res[0]
+
 
 class FindOne(FindQuery[FindQueryResultType]):
     """
@@ -643,6 +696,7 @@ class FindOne(FindQuery[FindQueryResultType]):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> "FindOne[FindQueryResultType]":
         ...
 
@@ -654,6 +708,7 @@ class FindOne(FindQuery[FindQueryResultType]):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> "FindOne[FindQueryProjectionType]":
         ...
 
@@ -664,6 +719,7 @@ class FindOne(FindQuery[FindQueryResultType]):
         session: Optional[ClientSession] = None,
         ignore_cache: bool = False,
         fetch_links: bool = False,
+        **pymongo_kwargs,
     ) -> Union[
         "FindOne[FindQueryResultType]", "FindOne[FindQueryProjectionType]"
     ]:
@@ -674,6 +730,7 @@ class FindOne(FindQuery[FindQueryResultType]):
         :param projection_model: Optional[Type[BaseModel]] - projection model
         :param session: Optional[ClientSession] - pymongo session
         :param ignore_cache: bool
+        :param **pymongo_kwargs: pymongo native parameters for find operation (if Document class contains links, this parameter must fit the respective parameter of the aggregate MongoDB function)
         :return: FindOne - query instance
         """
         self.find_expressions += args  # type: ignore # bool workaround
@@ -681,6 +738,7 @@ class FindOne(FindQuery[FindQueryResultType]):
         self.set_session(session=session)
         self.ignore_cache = ignore_cache
         self.fetch_links = fetch_links
+        self.pymongo_kwargs.update(pymongo_kwargs)
         return self
 
     def update_one(
@@ -688,6 +746,7 @@ class FindOne(FindQuery[FindQueryResultType]):
         *args: Mapping[str, Any],
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> UpdateOne:
         """
         Create [UpdateOne](https://roman-right.github.io/beanie/api/queries/#updateone) query using modifications and
@@ -698,13 +757,19 @@ class FindOne(FindQuery[FindQueryResultType]):
         """
         return cast(
             UpdateOne,
-            self.update(*args, session=session, bulk_writer=bulk_writer),
+            self.update(
+                *args,
+                session=session,
+                bulk_writer=bulk_writer,
+                **pymongo_kwargs,
+            ),
         )
 
     def delete_one(
         self,
         session: Optional[ClientSession] = None,
         bulk_writer: Optional[BulkWriter] = None,
+        **pymongo_kwargs,
     ) -> DeleteOne:
         """
         Provide search criteria to the [DeleteOne](https://roman-right.github.io/beanie/api/queries/#deleteone) query
@@ -715,7 +780,10 @@ class FindOne(FindQuery[FindQueryResultType]):
         # This is because delete may also return a DeleteOne type in general, and mypy can not be sure in this case
         # See https://mypy.readthedocs.io/en/stable/common_issues.html#narrowing-and-inner-functions
         return cast(
-            DeleteOne, self.delete(session=session, bulk_writer=bulk_writer)
+            DeleteOne,
+            self.delete(
+                session=session, bulk_writer=bulk_writer, **pymongo_kwargs
+            ),
         )
 
     async def replace_one(
@@ -758,17 +826,21 @@ class FindOne(FindQuery[FindQueryResultType]):
             return None
 
     async def _find_one(self):
-        if self.fetch_links:
-            lookup_queries = construct_lookup_queries(self.document_model)
-            result = (
-                await self.document_model.find(*self.find_expressions)
-                .aggregate(
-                    aggregation_pipeline=lookup_queries,
-                    projection_model=self.projection_model,
-                    session=self.session,
+        if self.document_model.get_link_fields() is not None:
+            for i, query in enumerate(self.find_expressions):
+                self.find_expressions[i] = convert_ids(
+                    query,
+                    doc=self.document_model,
+                    fetch_links=self.fetch_links,
                 )
-                .to_list(length=1)
-            )
+
+        if self.fetch_links:
+            result = await self.document_model.find(
+                *self.find_expressions,
+                session=self.session,
+                fetch_links=self.fetch_links,
+                **self.pymongo_kwargs,
+            ).to_list(length=1)
             if result:
                 return result[0]
             else:
@@ -777,6 +849,7 @@ class FindOne(FindQuery[FindQueryResultType]):
             filter=self.get_filter_query(),
             projection=get_projection(self.projection_model),
             session=self.session,
+            **self.pymongo_kwargs,
         )
 
     def __await__(
